# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime
import logging
from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Splitted, String

logger = logging.getLogger("driverng.cfx_neoplex")


class CfxNeoplexBase(Splitted):
    SEPARATOR = ","


class CfxNeoplexResult(object):
    FIELDS = [
        String("no"),
        String("name"),
        String("well"),
        String("sample_type"),
        String("fam"),
        String("empty_1"),
        String("empty_2"),
        String("hex"),
        String("empty_3"),
        String("empty_4"),
        String("cal_red_610"),
        String("empty_5"),
        String("empty_6"),
        String("quasar_670"),
        String("empty_7"),
        String("empty_8"),
        String("quasar_705"),
        String("empty_9"),
        String("empty_10"),
        String("ic"),
        String("result"),
        String("comment"),
        String("empty_11"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for idx, elt in enumerate(line):
            field = fields[idx]
            setattr(self, field.name, elt)


class CfxNeoplexResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: CfxNeoplexResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, skipinitialspace=True)
        self.analysis_date = None
        for index, line in enumerate(list(reader)):
            if line[0] == "Analysis Date:":
                try:
                    self.analysis_date = datetime.datetime.strptime(
                        line[1].strip(), "%Y-%m-%d %H:%M:%S"
                    )
                except ValueError as e:
                    logger.info("La date des résultats n'est pas au bon format %s", e)
            # remove the 5 first lines
            if index in [0, 1, 2, 3, 4]:
                continue
            obj = self.add(type_message=MessageV2.RESULT)
            obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "cfx_neoplex", "label": "CFX Logiciel NeoPlex"}

    def parse(self, data):
        obj = CfxNeoplexResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        detected = False
        RESULTS = ()

        for m in message:
            fields = (
                m.fam,
                m.empty_1,
                m.empty_2,
                m.hex,
                m.empty_3,
                m.empty_4,
                m.cal_red_610,
                m.empty_5,
                m.empty_6,
                m.quasar_670,
                m.empty_7,
                m.empty_8,
                m.quasar_705,
                m.empty_9,
                m.empty_10,
                m.ic,
                m.result,
                m.comment,
            )
            sample_id = rack = ""
            if m.name and m.name != "Name":
                sample_id = m.name
                rack = m.well
            if m.name == "Name":
                NAMES = fields
            if sample_id and m.sample_type != "SAMPLE":
                if m.result != "Not detected (IC Invalid)":
                    detected = True
                    control_id = "{}_{}".format(sample_id, m.no)
                    sample = worklist.get_or_create(
                        sample_id=control_id,
                        control_lot=control_id,
                        rack=rack,
                        type="control",
                    )
                    CODES = fields
            elif sample_id and sample_id != "/" and m.sample_type == "SAMPLE":
                if m.result != "Not detected (IC Invalid)":
                    detected = True
                    sample = worklist.get_or_create(
                        sample_id=sample_id, rack=rack, type="patient"
                    )
                    CODES = fields
            elif not sample_id and detected:
                RESULTS = fields
                for i in range(len(CODES)):
                    if CODES[i] and RESULTS[i]:
                        sample.results.add(
                            code=CODES[i], value=RESULTS[i], date=message.analysis_date
                        )
                    if CODES[i] and not RESULTS[i]:
                        sample.results.add(
                            code=NAMES[i], value=CODES[i], date=message.analysis_date
                        )
                detected = False

        return worklist
