# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "alinity"


def test_encode_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    message = self.order(
        worklist=worklist,
        context={"control_id": "1521651964637", "placer_order_number": "1234547"},
    )
    expected = """
    MSH|^~\\&|LISApp|MainFacility|AlinityApp|LabFacility|20130715141912||OML^O33^OML_O33|1521651964637|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-28^IHE
    PID|||036666666||NYME^ANO^^^^^L||20110513|F
    PV1||N|^LOC
    SPM|1|||""|||||||P^Patient^HL70369||||||20130513114242
    SAC|||123456||||||||||||||||||||||||||1:10
    ORC|NW||||||||20130603110342
    TQ1|1||||||||S^STAT^HL70485
    OBR||1234547||A^A^99ABT||||||||||||PRESCRIBER
    SPM|1|||""|||||||P^Patient^HL70369||||||20130513114242
    SAC|||123456
    ORC|NW||||||||20130603110342
    TQ1|1||||||||S^STAT^HL70485
    OBR||1234547||B^B^99ABT||||||||||||PRESCRIBER
    SPM|1|||""|||||||P^Patient^HL70369||||||20130513114242
    SAC|||123456
    ORC|CA||||||||20130603110342
    TQ1|1||||||||S^STAT^HL70485
    OBR||1234547||C^C^99ABT||||||||||||PRESCRIBER
    """
    assert message == expected


def test_decode_result_exception(self):
    self.configure(alinity_code_exception="ERREUR_EXCEPTION")
    worklist = self.decode(
        """
    MSH|^~\\&|ALINITY1|CATHERINE|MCA|CATHERINE|20181018205014+0200||OUL^R22^OUL_R22|369656b1-6cc2-44d4-94f3-736444c026f2|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    PID|||1234||ANO^NYME^^^^^L||19790409|M
    SPM|1|||""|||||||P^Patient^HL70369||||||20181018105000
    SAC|||670118106633|||||||M2053|6||||2^Position^99ABT~2^Module^99ABT
    OBR||1539878536060||355^Total PSA^99ABT||||||||||||301047353
    ORC|SC||||CM
    TQ1|||||||||R^Routine^HL70485
    OBX|1|CE|355^Total PSA^99ABT|3|1044^Impossible de calculer le resultat.La valeur URL finale se situe hors des limites definies pour le calibrateur le plus bas.^99ABT|||""|||X|||||FSE~FSE||Alinity ci-series^Abbott~Ai01641^Abbott|20181018191259||||||||||RSLT
    TCD|355^Total PSA^99ABT||||||||||1:10^^99ABT
    INV|355-1^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|00263^^99ABT||||||||20190205||||89011FN00
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "355",
                        "date": datetime.datetime(2018, 10, 18, 19, 12, 59),
                        "flags": [
                            "1044 Impossible de calculer le resultat.La valeur URL finale se situe hors des limites definies pour le calibrateur le plus bas."
                        ],
                        "instrument": "Ai01641",
                        "dilution": "1:10",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2019, 2, 5, 0, 0),
                                "lot": "89011FN00-00263",
                                "type": "reagent",
                            }
                        ],
                        "operator": "FSE",
                        "unit": "",
                        "value": "ERREUR_EXCEPTION",
                    }
                ],
                "rack": "M2053-6",
                "sample_id": "670118106633",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_exception_with_spm(self):
    self.configure(alinity_code_exception="ERREUR_EXCEPTION")
    worklist = self.decode(
        """
    MSH|^~\\&|ALINITY 1|BIOCHIMIE|MCA|LA FARLEDE|20230106080000+0100||OUL^R22^OUL_R22|606b6afc-7fdf-4910-bd47-8529f7ec11e8|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    SPM|1|||""|||||||P^Patient^HL70369
    SAC|||991030106001|||||||K9825|4||||17^Position^99ABT~1^Module^99ABT
    OBR||""||468^hsTnI STAT^99ABT
    ORC|SC||||CM
    TQ1|||||||||R^Routine^HL70485
    OBX|1|CE|468^hsTnI STAT^99ABT|1|3424^Erreur d'aspiration du pipeteur (Sample ).^99ABT|||""|||X|||||LGU~LGU||Alinity ci-series^Abbott~AI23972^Abbott|20230106080000||||||||||RSLT
    TCD|468^hsTnI STAT^99ABT||||||||||UNDILUTED^^99ABT
    INV|468-1^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|01717^^99ABT||||||||20230228||||44253UD00
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "468",
                        "date": datetime.datetime(2023, 1, 6, 8, 0),
                        "flags": ["3424 Erreur d'aspiration du pipeteur (Sample )."],
                        "instrument": "AI23972",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2023, 2, 28, 0, 0),
                                "lot": "44253UD00-01717",
                                "type": "reagent",
                            }
                        ],
                        "operator": "LGU",
                        "unit": "",
                        "value": "ERREUR_EXCEPTION",
                    }
                ],
                "sample_id": "991030106001",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_without_patient(self):
    worklist = self.decode(
        """
    MSH|^~\\&|ALINITY|LAPEYRONIE|MCA|LAPEYRONIE|20190321160541+0100||OUL^R22^OUL_R22|b4d4f06d-451c-4bc6-85e2-46510838ddd0|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    SPM|1|||""|||||||P^Patient^HL70369
    SAC|||630010800975|||||||P0717|2||||10^Position^99ABT~1^Module^99ABT
    OBR||""||729^Anti-HBc^99ABT
    ORC|SC||||CM
    TQ1|||||||||R^Routine^HL70485
    OBX|1|ST|729^Anti-HBc^99ABT|1|0.07|{s_co_ratio}^S/CO^UCUM||""|||F|||||CORINNE~CORINNE||Alinity ci-series^Abbott~Ai01834^Abbott|20190321143534||||||||||RSLT
    TCD|729^Anti-HBc^99ABT||||||||||UNDILUTED^^99ABT
    INV|729^^99ABT|NA^Not applicable^HL70383|CA^Calibration^99ABT||||||||||20190207113850|||88324LI00
    INV|729-1^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|00027^^99ABT||||||||20190426||||88324LI00
    OBX|2|ST|729.I^Anti-HBc^99ABT|1|Nonreactive|||""|||F|||||CORINNE~CORINNE||Alinity ci-series^Abbott~Ai01834^Abbott|20190321143534||||||||||RSLT
    OBX|3|ST|729.R^Anti-HBc^99ABT^S_RAW^Raw Supplemental^IHELAW|1|385|^RLU||""|||F|||||CORINNE~CORINNE||Alinity ci-series^Abbott~Ai01834^Abbott|20190321143534||||||||||RSLT
    OBX|4|EI|729.G^Anti-HBc^99ABT^S_OTHER^Other Supplemental^IHELAW|1|a5cc7568-bd51-4817-a591-b3496dadeabd|||""|||F|||||CORINNE~CORINNE||Alinity ci-series^Abbott~Ai01834^Abbott|20190321143534||||||||||RSLT
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "729",
                        "date": datetime.datetime(2019, 3, 21, 14, 35, 34),
                        "instrument": "Ai01834",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2019, 4, 26, 0, 0),
                                "lot": "88324LI00-00027",
                                "type": "reagent",
                            }
                        ],
                        "operator": "CORINNE",
                        "unit": "{s_co_ratio}",
                        "value": "0.07",
                        "flags": [],
                    },
                    {
                        "code": "729.I",
                        "date": datetime.datetime(2019, 3, 21, 14, 35, 34),
                        "instrument": "Ai01834",
                        "operator": "CORINNE",
                        "unit": "",
                        "value": "Nonreactive",
                        "flags": [],
                    },
                    {
                        "code": "729.R",
                        "date": datetime.datetime(2019, 3, 21, 14, 35, 34),
                        "instrument": "Ai01834",
                        "operator": "CORINNE",
                        "unit": "",
                        "value": "385",
                        "flags": [],
                    },
                    {
                        "code": "729.G",
                        "date": datetime.datetime(2019, 3, 21, 14, 35, 34),
                        "instrument": "Ai01834",
                        "operator": "CORINNE",
                        "unit": "",
                        "value": "a5cc7568-bd51-4817-a591-b3496dadeabd",
                        "flags": [],
                    },
                ],
                "sample_id": "630010800975",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result(self):
    date = datetime.datetime(2018, 6, 19, 14, 23, 58)
    worklist = self.decode(
        """
    MSH|^~\\&|Alinity-1cc-SCM01258|Deux Rives|DMS|Deux Rives|20180619142406+0200||OUL^R22^OUL_R22|eae91559-09ea-45c3-b96c-1612e6b6d6ed|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    PID|||*******||**********^*********^^^^^*||********|*
    PV1||*|^****
    SPM|1|||""|||||||P^Patient^HL70369||||||20180619103300
    SAC|||041806119250|||||||LAS|1
    OBR||60245e679fc176ee696836dfc288c840||1005^Urea^99ABT
    ORC|SC||||IP
    TQ1|||||||||R^Routine^HL70485
    OBX|1|ST|1005^Urea^99ABT|1|8.7|mmol/L^mmol/L^UCUM|3.6 - 7.1|HIGH^^99ABT~CNTL^^99ABT|||F|||||Admin~Admin||Alinity ci-series^Abbott~AC01132^Abbott|20180619142358||||||||||RSLT
    TCD|1005^Urea^99ABT||||||||||STANDARD^^99ABT
    INV|1005^^99ABT|NA^Not applicable^HL70383|CA^Calibration^99ABT|||||||||20180626093433|20180619093433|||98734UN17-01337
    INV|UREA0-1^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|01337^^99ABT||||||||20181026||||98734UN17-01337
    OBX|2|ST|1005.R^Urea^99ABT^S_RAW^Raw Supplemental^IHELAW|1|-0.0498|^Abs||""|||F|||||Admin~Admin||Alinity ci-series^Abbott~AC01132^Abbott|20180619142358||||||||||RSLT
    OBX|3|EI|1005.G^Urea^99ABT^S_OTHER^Other Supplemental^IHELAW|1|2a168d21-d779-420b-b7a9-d111b7658d9f|||""|||F|||||Admin~Admin||Alinity ci-series^Abbott~AC01132^Abbott|20180619142358||||||||||RSLT
    INV|HILS0-1^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|0001^^99ABT||||||||20190603||||00001
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "1005",
                        "flags": ["HIGH", "CNTL"],
                        "instrument": "AC01132",
                        "operator": "Admin",
                        "dilution": "",
                        "unit": "mmol/L",
                        "value": "8.7",
                        "date": date,
                        "lots": [
                            {
                                "expiration": datetime.datetime(2018, 10, 26, 0, 0),
                                "lot": "98734UN17-01337-01337",
                                "type": "reagent",
                            }
                        ],
                    },
                    {
                        "code": "1005.R",
                        "flags": [],
                        "instrument": "AC01132",
                        "operator": "Admin",
                        "dilution": "",
                        "unit": "",
                        "value": "-0.0498",
                        "date": date,
                    },
                    {
                        "code": "1005.G",
                        "flags": [],
                        "instrument": "AC01132",
                        "operator": "Admin",
                        "unit": "",
                        "dilution": "",
                        "value": "2a168d21-d779-420b-b7a9-d111b7658d9f",
                        "date": date,
                        "lots": [
                            {
                                "expiration": datetime.datetime(2019, 6, 3, 0, 0),
                                "lot": "00001-0001",
                                "type": "reagent",
                            }
                        ],
                    },
                ],
                "sample_id": "041806119250",
                "rack": "LAS-1",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_trame_disponibilty(self):
    worklist = self.decode(
        """
    MSH|^~\\&|Alinity-1cc-SCM01258|Deux Rives|DMS|Deux Rives|20180619142408+0200||TCU^U10^TCU_U10|ec013965-01f6-48e2-b2cb-cfca48a2b4d7|P|2.5.1|||NE|AL||UNICODE UTF-8
    EQU|Alinity ci-series^Abbott~SCM01258^Abbott|20180619142406
    TCC|1020^AlkP^99ABT|1020||||||||||||A^Available^99ABT
    TCC|1101^Na-C^99ABT|1101||||||||||||A^Available^99ABT
    TCC|1102^K-C^99ABT|1102||||||||||||A^Available^99ABT
    TCC|1103^Cl-C^99ABT|1103||||||||||||A^Available^99ABT
    TCC|1133^Na-CU^99ABT|1133||||||||||||A^Available^99ABT
    TCC|1134^K-CU^99ABT|1134||||||||||||A^Available^99ABT
    TCC|1135^Cl-CU^99ABT|1135||||||||||||A^Available^99ABT
    """
    )
    expected = {"context": "result", "samples": []}
    assert worklist == expected


def test_decode_controle_quality(self):
    date = datetime.datetime(2016, 8, 1, 14, 13, 45)
    worklist = self.decode(
        """
    MSH|^~\\&|Alinity ci|Lab2|LIS|MainLab|20160801141327||OUL^R22^OUL_R22|5828c7c3-69d1-437b-92e7-8ed6865e1d61|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    SPM|1|||""|||||||Q^Control specimen^HL70369
    SAC|||123BHCG1|||||||T1040|1||||4^Position^99ABT~1^Module^99ABT
    INV|123BHCG1^^99ABT|NA^Not applicable^HL70383|CO^Assay Control^HL70384|32123^^99ABT||||||||20201122||||45333DD11
    OBR||""||29^B-hCG^99ABT
    ORC|SC||||CM
    TQ1|||||||||R^Routine^HL70485
    OBX|1|ST|29^B-hCG^99ABT|1|310.26|^mIU/mL|300 - 400|RUO^^99ABT|||F|||||Admin~Admin||Alinity ci-series^Abbott~i00002|20160801141345||||||||||RSLT
    TCD|29^B-hCG^99ABT||||||||||UNDILUTED^^99ABT
    INV|29^^99ABT|NA^Not applicable^HL70383|CA^Calibration^99ABT||||||||||20160728170808|||65105BG51
    INV|29-1^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|88105^^99ABT||||||||||||65105BG51
    OBX|2|ST|29.R^B-hCG^99ABT^S_RAW^Raw Supplemental^IHELAW|1|90012|^RLU||""|||F|||||Admin~Admin||Alinity ci-series^Abbott~i00002^Abbott|20160801141345||||||||||RSLT
    OBX|3|EI|29.G^B-hCG^99ABT^S_OTHER^Other Supplemental^IHELAW|1|b8ce18eb-2f74-46ff-87c7-1b1bd3f52812|||""|||F|||||Admin~Admin||Alinity ci-series^Abbott~i00002^Abbott|20160801141345||||||||||RSLT
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "29",
                        "date": date,
                        "instrument": "i00002",
                        "lots": [
                            {
                                "expiration": None,
                                "lot": "65105BG51-88105",
                                "type": "reagent",
                            }
                        ],
                        "operator": "Admin",
                        "unit": "",
                        "value": "310.26",
                        "flags": [],
                    },
                    {
                        "code": "29.R",
                        "date": date,
                        "instrument": "i00002",
                        "operator": "Admin",
                        "unit": "",
                        "value": "90012",
                        "flags": [],
                    },
                    {
                        "code": "29.G",
                        "date": date,
                        "instrument": "i00002",
                        "operator": "Admin",
                        "unit": "",
                        "value": "b8ce18eb-2f74-46ff-87c7-1b1bd3f52812",
                        "flags": [],
                    },
                ],
                "sample_id": "123BHCG1",
                "control_lot": "123BHCG1",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    MSH|^~\\&|Alinity-1i-SCM01110|Labo XXX|DMS|Labo XXX|20180405155013+0200||QBP^Q11^QBP_Q11|7369025b-facd-46e6-ab72-1fe624104d53|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-27^IHE
    QPD|WOS^Work Order Step^IHELAW|19|041804604305
    RCP|I||R^Real Time^HL70394
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "041804604305"}],
    }
    assert worklist == expected


def test_void_order(self):
    message = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
            MSH|^~\\&|ALINITY2|CATHERINE|MCA|CATHERINE|20181016165241+0200||QBP^Q11^QBP_Q11|c5bdf30f-f4d4-453d-8c61-447455f711d5|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-27^IHE
            QPD|WOS^Work Order Step^IHELAW|449|640116108096
            RCP|I||R^Real Time^HL70394
            """
            ),
            "control_id": "20130715141912",
        },
    )
    expected = """
    MSH|^~\\&|LISApp|MainFacility|AlinityApp|LabFacility|20130715141912||OML^O33^OML_O33|20130715141912|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-28^IHE
    SPM|1|||""|||||||U^Unknow^HL70369
    SAC|||640116108096
    ORC|DC||||||||20130715141912"""
    assert message == expected
