# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "bactec9000"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&
    P|1||036666666||NYME^ANO||20110513|F||||||||||||||||||||||||LOC
    O|1|123456|||||20130513114242|||||||20130603110342|^Serum|PRESCRIBER
    L|1
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&
    Q|1|^MB941660443||||||||||O
    L|1
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "MB941660443"}],
    }
    assert worklist == expected


def test_decode_result(self):
    worklist = self.decode(
        """
    H|\\^&
    P|1||211-45-9687||SAM^HAPSHIRE|||||||||||||||||||||||||||
    O|1|A00006|||||19980109100700
    R|1|^^^440600000337^06|BACTEC_POS|||||P|||20160109113911||1/A01
    R|2|^^^449000000341^90|BACTEC_NEG|||||P|||20160109113911||1/A17
    R|3|^^^440600000348^06|BACTEC_ONGOING|||||P|||20160109113911|20160109113911|1/A18
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "A00006",
                "rack": "A01",
                "results": [
                    {
                        "code": "HEMOC1",
                        "date": None,
                        "instrument": "1",
                        "value": "BACTEC_POS",
                    },
                    {
                        "code": "ID_BOTTLE1",
                        "date": None,
                        "instrument": "1",
                        "value": "440600000337",
                    },
                    {
                        "code": "MILIEU_CULTURE1",
                        "date": None,
                        "instrument": "1",
                        "value": "06",
                    },
                ],
            },
            {
                "type": "patient",
                "sample_id": "A00006",
                "rack": "A17",
                "results": [
                    {
                        "code": "HEMOC2",
                        "date": None,
                        "instrument": "1",
                        "value": "BACTEC_NEG",
                    },
                    {
                        "code": "ID_BOTTLE2",
                        "date": None,
                        "instrument": "1",
                        "value": "449000000341",
                    },
                    {
                        "code": "MILIEU_CULTURE2",
                        "date": None,
                        "instrument": "1",
                        "value": "90",
                    },
                ],
            },
            {
                "type": "patient",
                "sample_id": "A00006",
                "rack": "A18",
                "results": [
                    {
                        "code": "HEMOC3",
                        "date": datetime.datetime(2016, 1, 9, 11, 39, 11),
                        "instrument": "1",
                        "value": "BACTEC_ONGOING",
                    },
                    {
                        "code": "ID_BOTTLE3",
                        "date": datetime.datetime(2016, 1, 9, 11, 39, 11),
                        "instrument": "1",
                        "value": "440600000348",
                    },
                    {
                        "code": "MILIEU_CULTURE3",
                        "date": datetime.datetime(2016, 1, 9, 11, 39, 11),
                        "instrument": "1",
                        "value": "06",
                    },
                ],
            },
        ],
    }
    assert worklist == expected
    assert worklist == expected
