# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "bactec_fx"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&|||||||||||V1.0|20130715141912
    P|1||036666666||NYME^ANO|||||||||||||||||||||||||||LOC
    O|1|123456||^^^A|||20130513114242
    O|2|123456||^^^B|||20130513114242
    L|1
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&
    C|1|This is a request for POSITIVE, NEGATIVE AND ONGOING vials from the LIS
    Q|1|ALL^ALL||^ALL||||||||P
    L|1"""
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "ALL"}],
    }

    assert worklist == expected


def test_decode_result_hemoc(self):
    worklist = self.decode(
        """
    H|\\^&|||Becton Dickinson||||||||V1.0|20081017105509
    P|1
    O|1|ACC-001||^^^BACTECFX_GND|||19700101000000
    R|1|^^^GND^446500000000|INST_NEW|||||P|||20081017105508||BACTECFX^65^5^01^AG01
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "BACTECFX_GNDGND",
                        "comments": [],
                        "date": None,
                        "instrument": "BACTECFX-65",
                        "value": "INST_NEW",
                    },
                    {
                        "code": "BOTTLE_ID",
                        "date": None,
                        "instrument": "BACTECFX-65",
                        "value": "446500000000",
                    },
                    {
                        "code": "BOTTLE_RACK",
                        "date": None,
                        "instrument": "BACTECFX-65",
                        "value": "AG01",
                    },
                    {"code": "INSTRUMENT", "date": None, "value": "BACTECFX-65"},
                    {
                        "code": "BACTECFX_GNDINTRODUCE_DATETIME",
                        "value": "2008/10/17 10:55:08",
                    },
                    {
                        "code": "BACTECFX_GNDBOTTLE_INTRODUCE_DATETIME",
                        "value": "20081017105508",
                    },
                    {"code": "BACTECFX_GNDINTRODUCE_DATE", "value": "2008/10/17"},
                    {"code": "BACTECFX_GNDINTRODUCE_TIME", "value": "10:55:08"},
                    {"code": "BACTECFX_GNDINTRODUCTION_DATE", "value": "17/10/2008"},
                    {"code": "BACTECFX_GNDINTRODUCTION_HEURE", "value": "10:55"},
                ],
                "sample_id": "ACC-001",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_hemoc_positive(self):
    worklist = self.decode(
        """
    H|\\^&|||Becton Dickinson||||||||V1.0|20081024064721
    P|1||PATID||LastName FirstName
    O|1|ACC-001||^^^BACTECFX_GND|||19700101000000
    R|1|^^^GND^446500000000|INST_POSITIVE|||||P|||20081017105508|20081017110031|BACTECFX^65^5^01^A-G01
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "BACTECFX_GNDGND",
                        "comments": [],
                        "date": datetime.datetime(2008, 10, 17, 11, 0, 31),
                        "instrument": "BACTECFX-65",
                        "value": "INST_POSITIVE",
                    },
                    {
                        "code": "BOTTLE_ID",
                        "date": datetime.datetime(2008, 10, 17, 11, 0, 31),
                        "instrument": "BACTECFX-65",
                        "value": "446500000000",
                    },
                    {
                        "code": "BOTTLE_RACK",
                        "date": datetime.datetime(2008, 10, 17, 11, 0, 31),
                        "instrument": "BACTECFX-65",
                        "value": "A-G01",
                    },
                    {
                        "code": "INSTRUMENT",
                        "date": datetime.datetime(2008, 10, 17, 11, 0, 31),
                        "value": "BACTECFX-65",
                    },
                    {
                        "code": "BACTECFX_GNDINTRODUCE_DATETIME",
                        "value": "2008/10/17 10:55:08",
                    },
                    {
                        "code": "BACTECFX_GNDBOTTLE_INTRODUCE_DATETIME",
                        "value": "20081017105508",
                    },
                    {"code": "BACTECFX_GNDINTRODUCE_DATE", "value": "2008/10/17"},
                    {"code": "BACTECFX_GNDINTRODUCE_TIME", "value": "10:55:08"},
                    {"code": "BACTECFX_GNDINTRODUCTION_DATE", "value": "17/10/2008"},
                    {"code": "BACTECFX_GNDINTRODUCTION_HEURE", "value": "10:55"},
                    {
                        "code": "BACTECFX_GNDDETERMINED_DATETIME",
                        "value": "20081017110031",
                    },
                    {"code": "BACTECFX_GNDDETERMINED_DATE", "value": "2008/10/17"},
                    {"code": "BACTECFX_GNDDETERMINED_TIME", "value": "11:00:31"},
                    {"code": "BACTECFX_GNDDATE_DETERMINATION", "value": "17/10/2008"},
                    {"code": "BACTECFX_GNDHEURE_DETERMINATION", "value": "11:00"},
                    {"code": "BACTECFX_GNDGROWTH_TIME", "value": "00H05"},
                    {
                        "code": "BACTECFX_GNDPOSITIVITY_DATETIME",
                        "value": "20081017110031",
                    },
                    {"code": "BACTECFX_GNDPOSITIVITY_DATE", "value": "2008/10/17"},
                    {"code": "BACTECFX_GNDPOSITIVITE_DATE", "value": "17/10/2008"},
                    {"code": "BACTECFX_GNDPOSITIVITY_TIME", "value": "11:00:31"},
                    {"code": "BACTECFX_GNDPOSITIVITE_HEURE", "value": "11:00"},
                    {
                        "code": "TIME_TO_DETECTION_POSITIVITY_BACTECFX_GND",
                        "value": "00H05",
                    },
                ],
                "sample_id": "ACC-001",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_hemoc_negative(self):
    worklist = self.decode(
        """
    H|\\^&|||Becton Dickinson||||||||V1.0|20081017110336
    P|1||PATID2||LastName2 FirstName2
    O|1|DAG||^^^BACTECFX_GND|||19700101000000
    R|1|^^^GND^446504178134|INST_NEGATIVE|||||P|||20080918155201|20080923160143|BACTECFX^65^28^01^A-A02
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "BACTECFX_GNDGND",
                        "comments": [],
                        "date": datetime.datetime(2008, 9, 23, 16, 1, 43),
                        "instrument": "BACTECFX-65",
                        "value": "INST_NEGATIVE",
                    },
                    {
                        "code": "BOTTLE_ID",
                        "date": datetime.datetime(2008, 9, 23, 16, 1, 43),
                        "instrument": "BACTECFX-65",
                        "value": "446504178134",
                    },
                    {
                        "code": "BOTTLE_RACK",
                        "date": datetime.datetime(2008, 9, 23, 16, 1, 43),
                        "instrument": "BACTECFX-65",
                        "value": "A-A02",
                    },
                    {
                        "code": "INSTRUMENT",
                        "date": datetime.datetime(2008, 9, 23, 16, 1, 43),
                        "value": "BACTECFX-65",
                    },
                    {
                        "code": "BACTECFX_GNDINTRODUCE_DATETIME",
                        "value": "2008/09/18 15:52:01",
                    },
                    {
                        "code": "BACTECFX_GNDBOTTLE_INTRODUCE_DATETIME",
                        "value": "20080918155201",
                    },
                    {"code": "BACTECFX_GNDINTRODUCE_DATE", "value": "2008/09/18"},
                    {"code": "BACTECFX_GNDINTRODUCE_TIME", "value": "15:52:01"},
                    {"code": "BACTECFX_GNDINTRODUCTION_DATE", "value": "18/09/2008"},
                    {"code": "BACTECFX_GNDINTRODUCTION_HEURE", "value": "15:52"},
                    {
                        "code": "BACTECFX_GNDDETERMINED_DATETIME",
                        "value": "20080923160143",
                    },
                    {"code": "BACTECFX_GNDDETERMINED_DATE", "value": "2008/09/23"},
                    {"code": "BACTECFX_GNDDETERMINED_TIME", "value": "16:01:43"},
                    {"code": "BACTECFX_GNDDATE_DETERMINATION", "value": "23/09/2008"},
                    {"code": "BACTECFX_GNDHEURE_DETERMINATION", "value": "16:01"},
                    {"code": "BACTECFX_GNDGROWTH_TIME", "value": "120H09"},
                ],
                "sample_id": "DAG",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
