# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "bd_max"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&||||||||||||20130715141912
    P|1||NYME
    O|1|123456||^^^BDMAX_AND|||||||||||A
    O|2|123456||^^^BDMAX_AND|||||||||||B
    L|1|N
    """
    assert message == expected


def test_encode_order_with_option_first_name(self):
    self.configure(bd_max_envoi_prenom=True)
    message = self.order()
    expected = """
    H|\\^&||||||||||||20130715141912
    P|1||NYME ANO
    O|1|123456||^^^BDMAX_AND|||||||||||A
    O|2|123456||^^^BDMAX_AND|||||||||||B
    L|1|N
    """
    assert message == expected


def test_encode_order_with_secondary_tube(self):
    self.configure(bd_max_tube_secondaire=True)
    message = self.order()
    expected = """
    H|\\^&||||||||||||20130715141912
    P|1||NYME
    O|1|987654321||^^^BDMAX_AND|||||||||||A
    O|2|987654321||^^^BDMAX_AND|||||||||||B
    L|1|N
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||Becton Dickinson||||||||V1.0|20140823151055
    Q|1|ALL^ALL||^CONFIRMED_POSITIVE||20111105120000|20111106223000|||||P
    L|1|N
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "ALL"}],
    }
    assert worklist == expected


def test_decode_result(self):
    worklist = self.decode(
        """
    H|\\^&|||Becton Dickinson||||||||V1.0|20140823151055
    P|1||12345678
    O|1| AB99990008||^^^BDMAX_AND
    R|1|^^^AND|INST_NEGATIVE|||||F|||20140823132809||BDMAX_AND^^^1^62-A4-A-BOT-10^^^SA|^FALSE
    R|2|^^^AND|INST_NEGATIVE|||||F|||20140823132809||BDMAX_AND^^^1^62-A4-A-BOT-10^^^MRSA|^FALSE
    R|3|^^^AND|INST_NEGATIVE|||||P|||20140823132809||BDMAX_AND^^^1^62-A4-A-BOT-10^^^TV|^FALSE
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "AB99990008",
                "results": [
                    {
                        "code": "ANDSA",
                        "date": datetime.datetime(2014, 8, 23, 13, 28, 9),
                        "value": "INST_NEGATIVE",
                    },
                    {
                        "code": "ANDMRSA",
                        "date": datetime.datetime(2014, 8, 23, 13, 28, 9),
                        "value": "INST_NEGATIVE",
                    },
                    {
                        "code": "ANDTV",
                        "date": datetime.datetime(2014, 8, 23, 13, 28, 9),
                        "value": "INST_NEGATIVE",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_ct_results(self):
    date = datetime.datetime(2023, 5, 2, 19, 18, 42)
    worklist = self.decode(
        """
    H|\\^&|||Becton Dickinson||||||||V1.0|20230502220354
    P|1||KK3050201090
    O|1|KK3050201090||^^^BDMAX_AND||||||||||B33226670020240427KH78|BD MAX EX ENT BAC 26
    R|1|^^^AND|INST_NEGATIVE|||||P|||20230502191842||BDMAX_AND^^^2^7670-A01-A-BOT-1^^^Shig|^FALSE
    R|2|^^^AND|INST_NEGATIVE|||||P|||20230502191842||BDMAX_AND^^^2^7670-A01-A-BOT-1^^^STX|^FALSE
    R|3|^^^AND|INST_NEGATIVE|||||P|||20230502191842||BDMAX_AND^^^2^7670-A01-A-BOT-1^^^Campy|^FALSE
    R|4|^^^AND|INST_NEGATIVE|||||P|||20230502191842||BDMAX_AND^^^2^7670-A01-A-BOT-1^^^Salm|^FALSE
    R|5|^^^AND|INST_NEGATIVE|||||P|||20230502191842||BDMAX_AND^^^2^7670-A01-A-BOT-1^^^Vibrio|^FALSE
    R|6|^^^AND|INST_NEGATIVE|||||P|||20230502191842||BDMAX_AND^^^2^7670-A01-A-BOT-1^^^Plesio|^FALSE
    R|7|^^^AND|INST_POSITIVE|||||P|||20230502191842||BDMAX_AND^^^2^7670-A01-A-BOT-1^^^ETEC|^FALSE
    R|8|^^^AND|INST_NEGATIVE|||||P|||20230502191842||BDMAX_AND^^^2^7670-A01-A-BOT-1^^^Yersi|^FALSE
    R|9|^^^CT|-1.00||||||||20230502191842||BDMAX_AND^^^2^7670-A01-A-BOT-1^^^ROX
    R|10|^^^CT|-1.00||||||||20230502191842||BDMAX_AND^^^2^7670-A01-A-BOT-1^^^CY5
    R|11|^^^CT|-1.00||||||||20230502191842||BDMAX_AND^^^2^7670-A01-A-BOT-1^^^FAM
    R|12|^^^CT|26.90||||||||20230502191842||BDMAX_AND^^^2^7670-A01-A-BOT-1^^^CY55
    R|13|^^^CT|25.70||||||||20230502191842||BDMAX_AND^^^2^7670-A01-A-BOT-1^^^VIC
    R|14|^^^CT|-1.00||||||||20230502191842||BDMAX_AND^^^2^7670-A01-A-TOP-1^^^ROX
    R|15|^^^CT|-1.00||||||||20230502191842||BDMAX_AND^^^2^7670-A01-A-TOP-1^^^CY5
    R|16|^^^CT|-1.00||||||||20230502191842||BDMAX_AND^^^2^7670-A01-A-TOP-1^^^FAM
    R|17|^^^CT|27.30||||||||20230502191842||BDMAX_AND^^^2^7670-A01-A-TOP-1^^^CY55
    R|18|^^^CT|-1.00||||||||20230502191842||BDMAX_AND^^^2^7670-A01-A-TOP-1^^^VIC
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "ANDShig", "date": date, "value": "INST_NEGATIVE"},
                    {"code": "ANDSTX", "date": date, "value": "INST_NEGATIVE"},
                    {"code": "ANDCampy", "date": date, "value": "INST_NEGATIVE"},
                    {"code": "ANDSalm", "date": date, "value": "INST_NEGATIVE"},
                    {"code": "ANDVibrio", "date": date, "value": "INST_NEGATIVE"},
                    {"code": "ANDPlesio", "date": date, "value": "INST_NEGATIVE"},
                    {"code": "ANDETEC", "date": date, "value": "INST_POSITIVE"},
                    {"code": "ANDYersi", "date": date, "value": "INST_NEGATIVE"},
                    {"code": "CTROX_BOT", "date": date, "value": "-1.00"},
                    {"code": "CTCY5_BOT", "date": date, "value": "-1.00"},
                    {"code": "CTFAM_BOT", "date": date, "value": "-1.00"},
                    {"code": "CTCY55_BOT", "date": date, "value": "26.90"},
                    {"code": "CTVIC_BOT", "date": date, "value": "25.70"},
                    {"code": "CTROX_TOP", "date": date, "value": "-1.00"},
                    {"code": "CTCY5_TOP", "date": date, "value": "-1.00"},
                    {"code": "CTFAM_TOP", "date": date, "value": "-1.00"},
                    {"code": "CTCY55_TOP", "date": date, "value": "27.30"},
                    {"code": "CTVIC_TOP", "date": date, "value": "-1.00"},
                ],
                "sample_id": "KK3050201090",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    date = datetime.datetime(2023, 2, 3, 10, 49, 2)
    worklist = self.decode(
        """
    H|\\^&|||Becton Dickinson||||||||V1.0|20230203140705
    P|1||QC123456N1
    O|1|LOT 2174286||^^^BDMAX_AND||||||||||B28210563820230812PP48|BD CTGCTV2 URINE 71
    R|1|^^^AND|INST_POSITIVE|||||P|||20230203104902||BDMAX_AND^^^1^3324-B10-B-TOP-6^^^CT|^CONFIRMED POSITIVE
    R|2|^^^AND|INST_POSITIVE|||||P|||20230203104902||BDMAX_AND^^^1^3324-B10-B-TOP-6^^^GC|^CONFIRMED POSITIVE
    R|3|^^^AND|INST_POSITIVE|||||P|||20230203104902||BDMAX_AND^^^1^3324-B10-B-TOP-6^^^TV|^CONFIRMED POSITIVE
    R|4|^^^CT|28.10||||||||20230203104902||BDMAX_AND^^^1^3324-B10-B-TOP-6^^^ROX
    R|5|^^^CT|27.40||||||||20230203104902||BDMAX_AND^^^1^3324-B10-B-TOP-6^^^CY5
    R|6|^^^CT|29.00||||||||20230203104902||BDMAX_AND^^^1^3324-B10-B-TOP-6^^^FAM
    R|7|^^^CT|29.90||||||||20230203104902||BDMAX_AND^^^1^3324-B10-B-TOP-6^^^CY55
    R|8|^^^CT|31.20||||||||20230203104902||BDMAX_AND^^^1^3324-B10-B-TOP-6^^^VIC
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "QC123456N1",
                "results": [
                    {"code": "ANDCT", "date": date, "value": "INST_POSITIVE"},
                    {"code": "ANDGC", "date": date, "value": "INST_POSITIVE"},
                    {"code": "ANDTV", "date": date, "value": "INST_POSITIVE"},
                    {"code": "CTROX_TOP", "date": date, "value": "28.10"},
                    {"code": "CTCY5_TOP", "date": date, "value": "27.40"},
                    {"code": "CTFAM_TOP", "date": date, "value": "29.00"},
                    {"code": "CTCY55_TOP", "date": date, "value": "29.90"},
                    {"code": "CTVIC_TOP", "date": date, "value": "31.20"},
                ],
                "sample_id": "QC123456N1",
                "type": "control",
            }
        ],
    }
    assert worklist == expected
