# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Worklist

parser_class = "bnpros"


def test_encode_order(self):
    expected = """
    H|\\^&||||||||||P|1|20130715141912
    P|1|036666666|||NYME^ANO|||F
    O|1|123456|123456|^^^A\\^^^B|S||||||||||||||||||||O
    L|1
    """
    message = self.order()
    assert message == expected


def test_decode_result(self):
    worklist = self.decode(
        """
    H|\\^&|||^010000^^H1P1O1R1C1Q1L1^0.9.4|||||||P|1|20131223150905
    P|1|||||
    O|1|191223001200|191223001200|^^^1^^400|R||||||||||||||||||||F
    R|1|^^^1^^400^^F|9.22|||||R||||20131223141741
    P|2|||||
    O|1|191223001200|191223001200|^^^2^^20|R||||||||||||||||||||F
    R|1|^^^2^^20^^F|2.2|||||R||||20131223141806
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "191223001200",
                "results": [
                    {
                        "code": "1",
                        "value": "9.22",
                        "date": datetime.datetime(2013, 12, 23, 14, 17, 41),
                    },
                    {
                        "code": "2",
                        "value": "2.2",
                        "date": datetime.datetime(2013, 12, 23, 14, 18, 6),
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    worklist = self.decode(
        """
    H|\\^&|||^010000^^H1P1O1R1C1Q1L1^0.9.4|||||||Q|1|20131223150905
    P|2|||||
    O|1|084744|084744|^^^2^^20|R||||||Q||||||||||||||F
    R|1|^^^2^^20^^F|2.2|||||R||||20131223141806
    L|1|F"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "control_lot": "084744",
                "sample_id": "084744",
                "results": [
                    {
                        "code": "2",
                        "value": "2.2",
                        "date": datetime.datetime(2013, 12, 23, 14, 18, 6),
                    }
                ],
            }
        ],
    }
    assert worklist == expected


_query_message = """
H|\\^&|||BNP^010000^^0.8.9|||||||P|1|20140404112549
Q|1|^141223006700||^^^ALL||||||||O
L|1"""


def test_decode_query(self):
    worklist = self.decode(_query_message)
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "141223006700"}],
    }
    assert worklist == expected


def test_encode_void_order(self):
    message = self.encode(
        Worklist(context="order"), context={"query": self.parse(_query_message)}
    )
    expected = """
    H|\\^&||||||||||P|1|20130715141912
    Q|1|^141223006700||^^^ALL||||||||X
    L|1"""
    assert message == expected
