# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy

parser_class = "lifecycle_perkinelmer"


def test_encode_order(self):
    _order = """036666666;NYME;ANO;;;;;;;20110513;58;;;;;;PRESCRIBER;;;;123456;20130513;;;;;;NJF;;20130603;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;"""
    message = self.order()
    assert message == _order


def test_encode_order_with_results(self):
    _order = """036666666;NYME;ANO;;;;;53;20000112;20110513;58;1;1;0;;;PRESCRIBER;;ECHO;;123456;20130513;20220215;;;;;NJF;;20130603;19920710;29;;20211212;20211224;0;;;;;;;;;;;;;;;;;;;;;;;;;;;;;"""
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].results.extend(
        [
            {"code": "FUMEUR", "value": "FUMEUR"},
            {"code": "ETHNIE", "value": "CAUCA"},
            {"code": "POIDS", "value": "62"},
            {"code": "DIABETE", "value": "OUI"},
            {"code": "ANTE_T21", "value": "NON"},
            {"code": "ECHOGRAPHISTE", "value": "ECHO"},
            {"code": "DDG", "value": "20000112"},
            {"code": "DE", "value": "20220215"},
            {"code": "TR21D_DDN", "value": "19920710"},
            {"code": "TR21T_AGE", "value": "29"},
            {"code": "TR21P_PREL", "value": "20211212"},
            {"code": "TR21P_TRANSF", "value": "20211224"},
            {"code": "TR21M_AMPT", "value": "0"},
        ]
    )
    message = self.order(worklist=worklist)
    assert message == _order


def test_decode_result_new_protocol(self):
    worklist = self.decode(
        """
    PATIENTID;LASTNAME;FIRSTNAME;19870910;;1;0;20210501;0;0;97;2107230442;20210723;;;150;6.68299077544094;;;;;150;0.0468262132987348;1.8;1.0253716105649291;;;83.8;;331;21;;34.36824;84;20210723;53;;;;;;;;24.1;2.3;2.3;0.6923658;20210723;106;1.76356;;;;;;1261;1175;8500;965;;;;;;;;;
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "NOMBRE_GEMELLAIRE", "value": "1"},
                    {"code": "DATE_PRELEVEMENT", "value": "23/07/2021"},
                    {"code": "ANTE_T21", "value": "0"},
                    {"code": "DATE_DEBUT_GROSSESSE", "value": "01/05/2021"},
                    {"code": "DIABETES_DID", "value": "0"},
                    {"code": "FUMEUR", "value": "0"},
                    {"code": "AGE_GESTATIONEL", "value": "97"},
                    {"code": "FIELD_RESULT_16", "value": "150"},
                    {"code": "FIELD_RESULT_17", "value": "6.68299077544094"},
                    {"code": "FIELD_RESULT_22", "value": "150"},
                    {"code": "FIELD_RESULT_23", "value": "0.0468262132987348"},
                    {"code": "FIELD_RESULT_24", "value": "1.8"},
                    {"code": "FIELD_RESULT_25", "value": "1.0253716105649291"},
                    {"code": "FIELD_RESULT_28", "value": "83.8"},
                    {"code": "FIELD_RESULT_30", "value": "331"},
                    {"code": "FIELD_RESULT_31", "value": "21"},
                    {"code": "FIELD_RESULT_33", "value": "34.36824"},
                    {"code": "FIELD_RESULT_34", "value": "84"},
                    {"code": "FIELD_RESULT_35", "value": "23/07/2021"},
                    {"code": "FIELD_RESULT_36", "value": "53"},
                    {"code": "FIELD_RESULT_44", "value": "24.1"},
                    {"code": "FIELD_RESULT_45", "value": "2.3"},
                    {"code": "FIELD_RESULT_46", "value": "2.3"},
                    {"code": "FIELD_RESULT_47", "value": "0.6923658"},
                    {"code": "FIELD_RESULT_48", "value": "20210723"},
                    {"code": "FIELD_RESULT_49", "value": "106"},
                    {"code": "FIELD_RESULT_50", "value": "1.76356"},
                    {"code": "FIELD_RESULT_56", "value": "1261"},
                    {"code": "FIELD_RESULT_57", "value": "1175"},
                    {"code": "FIELD_RESULT_58", "value": "8500"},
                    {"code": "FIELD_RESULT_59", "value": "965"},
                ],
                "sample_id": "2107230442",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_old_protocol(self):
    worklist = self.decode(
        """
    123435;Downing;Mary;19750723;20090515;1;0;20090601;0;0;102;1234567890;20090819;33.456;1.170994;65.7890;1.550994;100.0001;2.1717;33.456;1.140994; 1650.0909;1.8080;3.1;1.040987;;;65.3;;10000;2500;;35.34426;75.5;20090715;53;;;30.64;1; 20100510;20110405;4
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "DATE_DERNIERE_REGLE", "value": "15/05/2009"},
                    {"code": "NOMBRE_GEMELLAIRE", "value": "1"},
                    {"code": "DATE_PRELEVEMENT", "value": "19/08/2009"},
                    {"code": "ANTE_T21", "value": "0"},
                    {"code": "DATE_DEBUT_GROSSESSE", "value": "01/06/2009"},
                    {"code": "DIABETES_DID", "value": "0"},
                    {"code": "FUMEUR", "value": "0"},
                    {"code": "AGE_GESTATIONEL", "value": "102"},
                    {"code": "FIELD_RESULT_14", "value": "33.456"},
                    {"code": "FIELD_RESULT_15", "value": "1.170994"},
                    {"code": "FIELD_RESULT_16", "value": "65.7890"},
                    {"code": "FIELD_RESULT_17", "value": "1.550994"},
                    {"code": "FIELD_RESULT_18", "value": "100.0001"},
                    {"code": "FIELD_RESULT_19", "value": "2.1717"},
                    {"code": "FIELD_RESULT_20", "value": "33.456"},
                    {"code": "FIELD_RESULT_21", "value": "1.140994"},
                    {"code": "FIELD_RESULT_22", "value": "1650.0909"},
                    {"code": "FIELD_RESULT_23", "value": "1.8080"},
                    {"code": "FIELD_RESULT_24", "value": "3.1"},
                    {"code": "FIELD_RESULT_25", "value": "1.040987"},
                    {"code": "FIELD_RESULT_28", "value": "65.3"},
                    {"code": "FIELD_RESULT_30", "value": "10000"},
                    {"code": "FIELD_RESULT_31", "value": "2500"},
                    {"code": "FIELD_RESULT_33", "value": "35.34426"},
                    {"code": "FIELD_RESULT_34", "value": "75.5"},
                    {"code": "FIELD_RESULT_35", "value": "15/07/2009"},
                    {"code": "FIELD_RESULT_36", "value": "53"},
                    {"code": "FIELD_RESULT_39", "value": "30.64"},
                    {"code": "FIELD_RESULT_40", "value": "1"},
                    {"code": "FIELD_RESULT_41", "value": "20100510"},
                    {"code": "FIELD_RESULT_42", "value": "05/04/2011"},
                    {"code": "FIELD_RESULT_43", "value": "4"},
                ],
                "sample_id": "1234567890",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
